@@ -6,9 +6,9 @@
 /* SYNOPSIS
 /*	#include <dns.h>
 /*
-/*	DNS_RR	*dns_rr_create(name, rname, type, class, ttl, preference,
+/*	DNS_RR	*dns_rr_create(qname, rname, type, class, ttl, preference,
 /*				data, data_len)
-/*	const char *name;
+/*	const char *qname;
 /*	const char *rname;
 /*	unsigned short type;
 /*	unsigned short class;
@@ -31,7 +31,15 @@
 /*	DNS_RR	*list
 /*	int	(*compar)(DNS_RR *, DNS_RR *);
 /*
-/*	int	dns_rr_compare_pref(DNS_RR *a, DNS_RR *b)
+/*	int	dns_rr_compare_pref_ipv6(DNS_RR *a, DNS_RR *b)
+/*	DNS_RR	*list
+/*	DNS_RR	*list
+/*
+/*	int	dns_rr_compare_pref_ipv4(DNS_RR *a, DNS_RR *b)
+/*	DNS_RR	*list
+/*	DNS_RR	*list
+/*
+/*	int	dns_rr_compare_pref_any(DNS_RR *a, DNS_RR *b)
 /*	DNS_RR	*list
 /*	DNS_RR	*list
 /*
@@ -46,7 +54,7 @@
 /*	information, and maintain lists of DNS resource records.
 /*
 /*	dns_rr_create() creates and initializes one resource record.
-/*	The \fIname\fR field specifies the query name.
+/*	The \fIqname\fR field specifies the query name.
 /*	The \fIrname\fR field specifies the reply name.
 /*	\fIpreference\fR is used for MX records; \fIdata\fR is a null
 /*	pointer or specifies optional resource-specific data;
@@ -59,18 +67,20 @@
 /*
 /*	dns_rr_append() appends a resource record to a (list of) resource
 /*	record(s).
+/*	A null input list is explicitly allowed.
 /*
 /*	dns_rr_sort() sorts a list of resource records into ascending
 /*	order according to a user-specified criterion. The result is the
 /*	sorted list.
 /*
-/*	dns_rr_compare_pref() is a dns_rr_sort() helper to sort records
-/*	by their MX preference.
+/*	dns_rr_compare_pref_XXX() are dns_rr_sort() helpers to sort
+/*	records by their MX preference and by their address family.
 /*
 /*	dns_rr_shuffle() randomly permutes a list of resource records.
 /*
 /*	dns_rr_remove() removes the specified record from the specified list.
 /*	The updated list is the result value.
+/*	The record MUST be a list member.
 /* LICENSE
 /* .ad
 /* .fi
@@ -100,7 +110,7 @@
 
 /* dns_rr_create - fill in resource record structure */
 
-DNS_RR *dns_rr_create(const char *name, const char *rname,
+DNS_RR *dns_rr_create(const char *qname, const char *rname,
 		              ushort type, ushort class,
 		              unsigned int ttl, unsigned pref,
 		              const char *data, size_t data_len)
@@ -108,7 +118,7 @@ DNS_RR *dns_rr_create(const char *name, const char *rname,
     DNS_RR *rr;
 
     rr = (DNS_RR *) mymalloc(sizeof(*rr) + data_len - 1);
-    rr->name = mystrdup(name);
+    rr->qname = mystrdup(qname);
     rr->rname = mystrdup(rname);
     rr->type = type;
     rr->class = class;
@@ -128,7 +138,7 @@ void    dns_rr_free(DNS_RR *rr)
     if (rr) {
 	if (rr->next)
 	    dns_rr_free(rr->next);
-	myfree(rr->name);
+	myfree(rr->qname);
 	myfree(rr->rname);
 	myfree((char *) rr);
     }
@@ -146,7 +156,7 @@ DNS_RR *dns_rr_copy(DNS_RR *src)
      */
     dst = (DNS_RR *) mymalloc(len);
     memcpy((char *) dst, (char *) src, len);
-    dst->name = mystrdup(src->name);
+    dst->qname = mystrdup(src->qname);
     dst->rname = mystrdup(src->rname);
     dst->next = 0;
     return (dst);
@@ -164,9 +174,9 @@ DNS_RR *dns_rr_append(DNS_RR *list, DNS_RR *rr)
     return (list);
 }
 
-/* dns_rr_compare_pref - compare resource records by preference */
+/* dns_rr_compare_pref_ipv6 - compare records by preference, ipv6 preferred */
 
-int     dns_rr_compare_pref(DNS_RR *a, DNS_RR *b)
+int     dns_rr_compare_pref_ipv6(DNS_RR *a, DNS_RR *b)
 {
     if (a->pref != b->pref)
 	return (a->pref - b->pref);
@@ -181,6 +191,39 @@ int     dns_rr_compare_pref(DNS_RR *a, DNS_RR *b)
     return 0;
 }
 
+/* dns_rr_compare_pref_ipv4 - compare records by preference, ipv4 preferred */
+
+int     dns_rr_compare_pref_ipv4(DNS_RR *a, DNS_RR *b)
+{
+    if (a->pref != b->pref)
+	return (a->pref - b->pref);
+#ifdef HAS_IPV6
+    if (a->type == b->type)
+	return 0;
+    if (a->type == T_AAAA)
+	return (+1);
+    if (b->type == T_AAAA)
+	return (-1);
+#endif
+    return 0;
+}
+
+/* dns_rr_compare_pref_any - compare records by preference, protocol-neutral */
+
+int     dns_rr_compare_pref_any(DNS_RR *a, DNS_RR *b)
+{
+    if (a->pref != b->pref)
+	return (a->pref - b->pref);
+    return 0;
+}
+
+/* dns_rr_compare_pref - binary compatibility helper after name change */
+
+int     dns_rr_compare_pref(DNS_RR *a, DNS_RR *b)
+{
+    return (dns_rr_compare_pref_ipv6(a, b));
+}
+
 /* dns_rr_sort_callback - glue function */
 
 static int (*dns_rr_sort_user) (DNS_RR *, DNS_RR *);
