@@ -6,9 +6,10 @@
 /* SYNOPSIS
 /*	#include <smtp_stream.h>
 /*
-/*	void	smtp_timeout_setup(stream, timeout)
+/*	void	smtp_stream_setup(stream, timeout, enable_deadline)
 /*	VSTREAM *stream;
 /*	int	timeout;
+/*	int	enable_deadline;
 /*
 /*	void	smtp_printf(stream, format, ...)
 /*	VSTREAM *stream;
@@ -20,10 +21,11 @@
 /*	int	smtp_fgetc(stream)
 /*	VSTREAM *stream;
 /*
-/*	int	smtp_get(vp, stream, maxlen)
+/*	int	smtp_get(vp, stream, maxlen, flags)
 /*	VSTRING	*vp;
 /*	VSTREAM *stream;
 /*	ssize_t	maxlen;
+/*	int	flags;
 /*
 /*	void	smtp_fputs(str, len, stream)
 /*	const char *str;
@@ -43,16 +45,24 @@
 /*	VSTREAM *stream;
 /*	char	*format;
 /*	va_list	ap;
+/* LEGACY API
+/*	void	smtp_timeout_setup(stream, timeout)
+/*	VSTREAM *stream;
+/*	int	timeout;
+/*	int	enable_deadline;
 /* DESCRIPTION
 /*	This module reads and writes text records delimited by CR LF,
 /*	with error detection: timeouts or unexpected end-of-file.
 /*	A trailing CR LF is added upon writing and removed upon reading.
 /*
-/*	smtp_timeout_setup() arranges for a time limit on the smtp read
+/*	smtp_stream_setup() prepares the specified stream for SMTP read
 /*	and write operations described below.
 /*	This routine alters the behavior of streams as follows:
 /* .IP \(bu
-/*	The read/write timeout is set to the specified value.
+/*	When enable_deadline is non-zero, the stream is configured
+/*	to enforce a total time limit for each smtp_stream read/write
+/*	operation. Otherwise, the stream is configured to enforce
+/*	a time limit for each individual read/write system call.
 /* .IP \f(bu
 /*	The stream is configured to use double buffering.
 /* .IP \f(bu
@@ -72,6 +82,10 @@
 /*	and protects the program against running out of memory.
 /*	Specify a zero bound to turn off bounds checking.
 /*	The result is the last character read, or VSTREAM_EOF.
+/*	The \fIflags\fR argument is either SMTP_GET_FLAG_NONE (no
+/*	special processing) or SMTP_GET_FLAG_SKIP (skip over input
+/*	in excess of \fImaxlen\fR). Either way, a result value of
+/*	'\n' means that the input did not exceed \fImaxlen\fR.
 /*
 /*	smtp_fputs() writes its string argument to the named stream.
 /*	Long strings are not broken. Each string is followed by a
@@ -85,16 +99,22 @@
 /*	The stream is not flushed.
 /*
 /*	smtp_vprintf() is the machine underneath smtp_printf().
+/*
+/*	smtp_timeout_setup() is a backwards-compatibility interface
+/*	for programs that don't require per-record deadline support.
 /* DIAGNOSTICS
 /* .fi
 /* .ad
 /*	In case of error, a vstream_longjmp() call is performed to the
 /*	context specified with vstream_setjmp().
+/*	After write error, further writes to the socket are disabled.
+/*	This eliminates the need for clumsy code to avoid unwanted
+/*	I/O while shutting down a TLS engine or closing a VSTREAM.
 /*	Error codes passed along with vstream_longjmp() are:
 /* .IP SMTP_ERR_EOF
 /*	An I/O error happened, or the peer has disconnected unexpectedly.
 /* .IP SMTP_ERR_TIME
-/*	The time limit specified to smtp_timeout_setup() was exceeded.
+/*	The time limit specified to smtp_stream_setup() was exceeded.
 /* .PP
 /*	Additional error codes that may be used by applications:
 /* .IP SMTP_ERR_QUIET
@@ -102,6 +122,9 @@
 /*	the application.
 /*	This error is never generated by the smtp_stream(3) module, but
 /*	is defined for application-specific use.
+/* .IP SMTP_ERR_DATA
+/*	Application data error - the program cannot proceed with this
+/*	SMTP session.
 /* .IP SMTP_ERR_NONE
 /*	A non-error code that makes setjmp()/longjmp() convenient
 /*	to use.
@@ -146,28 +169,56 @@
 
 #include "smtp_stream.h"
 
-/* smtp_timeout_reset - reset per-stream timeout flag */
+/* smtp_timeout_reset - reset per-stream error flags, restart deadline timer */
 
 static void smtp_timeout_reset(VSTREAM *stream)
 {
     vstream_clearerr(stream);
+
+    /*
+     * Important: the time limit feature must not introduce any system calls
+     * when the input is already in the buffer, or when the output still fits
+     * in the buffer. Such system calls would really hurt when receiving or
+     * sending body content one line at a time.
+     */
+    if (vstream_fstat(stream, VSTREAM_FLAG_DEADLINE))
+	vstream_control(stream, VSTREAM_CTL_START_DEADLINE, VSTREAM_CTL_END);
 }
 
-/* smtp_timeout_detect - test the per-stream timeout flag */
+/* smtp_longjmp - raise an exception */
 
-static void smtp_timeout_detect(VSTREAM *stream)
+static NORETURN smtp_longjmp(VSTREAM *stream, int err, const char *context)
 {
-    if (vstream_ftimeout(stream))
-	vstream_longjmp(stream, SMTP_ERR_TIME);
+
+    /*
+     * If we failed to write, don't bang our head against the wall another
+     * time when closing the stream. In the case of SMTP over TLS, poisoning
+     * the socket with shutdown() is more robust than purging the VSTREAM
+     * buffer or replacing the write function pointer with dummy_write().
+     */
+    if (msg_verbose)
+	msg_info("%s: %s", context, err == SMTP_ERR_TIME ? "timeout" : "EOF");
+    if (vstream_wr_error(stream))
+	/* Don't report ECONNRESET (hangup), EINVAL (already shut down), etc. */
+	(void) shutdown(vstream_fileno(stream), SHUT_WR);
+    vstream_longjmp(stream, err);
 }
 
-/* smtp_timeout_setup - configure timeout trap */
+/* smtp_stream_setup - configure timeout trap */
 
-void    smtp_timeout_setup(VSTREAM *stream, int maxtime)
+void    smtp_stream_setup(VSTREAM *stream, int maxtime, int enable_deadline)
 {
+    const char *myname = "smtp_stream_setup";
+
+    if (msg_verbose)
+	msg_info("%s: maxtime=%d enable_deadline=%d",
+		 myname, maxtime, enable_deadline);
+
     vstream_control(stream,
 		    VSTREAM_CTL_DOUBLE,
 		    VSTREAM_CTL_TIMEOUT, maxtime,
+		    enable_deadline ? VSTREAM_CTL_START_DEADLINE
+		    : VSTREAM_CTL_STOP_DEADLINE,
 		    VSTREAM_CTL_EXCEPT,
 		    VSTREAM_CTL_END);
 }
@@ -183,16 +234,14 @@ void    smtp_flush(VSTREAM *stream)
      */
     smtp_timeout_reset(stream);
     err = vstream_fflush(stream);
-    smtp_timeout_detect(stream);
 
     /*
      * See if there was a problem.
      */
-    if (err != 0) {
-	if (msg_verbose)
-	    msg_info("smtp_flush: EOF");
-	vstream_longjmp(stream, SMTP_ERR_EOF);
-    }
+    if (vstream_ftimeout(stream))
+	smtp_longjmp(stream, SMTP_ERR_TIME, "smtp_flush");
+    if (err != 0)
+	smtp_longjmp(stream, SMTP_ERR_EOF, "smtp_flush");
 }
 
 /* smtp_vprintf - write one line to SMTP peer */
@@ -208,16 +257,14 @@ void    smtp_vprintf(VSTREAM *stream, const char *fmt, va_list ap)
     vstream_vfprintf(stream, fmt, ap);
     vstream_fputs("\r\n", stream);
     err = vstream_ferror(stream);
-    smtp_timeout_detect(stream);
 
     /*
      * See if there was a problem.
      */
-    if (err != 0) {
-	if (msg_verbose)
-	    msg_info("smtp_vprintf: EOF");
-	vstream_longjmp(stream, SMTP_ERR_EOF);
-    }
+    if (vstream_ftimeout(stream))
+	smtp_longjmp(stream, SMTP_ERR_TIME, "smtp_vprintf");
+    if (err != 0)
+	smtp_longjmp(stream, SMTP_ERR_EOF, "smtp_vprintf");
 }
 
 /* smtp_printf - write one line to SMTP peer */
@@ -242,22 +289,20 @@ int     smtp_fgetc(VSTREAM *stream)
      */
     smtp_timeout_reset(stream);
     ch = VSTREAM_GETC(stream);
-    smtp_timeout_detect(stream);
 
     /*
      * See if there was a problem.
      */
-    if (vstream_feof(stream) || vstream_ferror(stream)) {
-	if (msg_verbose)
-	    msg_info("smtp_fgetc: EOF");
-	vstream_longjmp(stream, SMTP_ERR_EOF);
-    }
+    if (vstream_ftimeout(stream))
+	smtp_longjmp(stream, SMTP_ERR_TIME, "smtp_fgetc");
+    if (vstream_feof(stream) || vstream_ferror(stream))
+	smtp_longjmp(stream, SMTP_ERR_EOF, "smtp_fgetc");
     return (ch);
 }
 
 /* smtp_get - read one line from SMTP peer */
 
-int     smtp_get(VSTRING *vp, VSTREAM *stream, ssize_t bound)
+int     smtp_get(VSTRING *vp, VSTREAM *stream, ssize_t bound, int flags)
 {
     int     last_char;
     int     next_char;
@@ -303,6 +348,7 @@ int     smtp_get(VSTRING *vp, VSTREAM *stream, ssize_t bound)
 	while (VSTRING_LEN(vp) > 0 && vstring_end(vp)[-1] == '\r')
 	    vstring_truncate(vp, VSTRING_LEN(vp) - 1);
 	VSTRING_TERMINATE(vp);
+	/* FALLTRHOUGH */
 
 	/*
 	 * Partial line: just read the remainder later. If we ran into EOF,
@@ -311,17 +357,24 @@ int     smtp_get(VSTRING *vp, VSTREAM *stream, ssize_t bound)
     default:
 	break;
     }
-    smtp_timeout_detect(stream);
+
+    /*
+     * Optionally, skip over excess input, protected by the same time limit.
+     */
+    if (last_char != '\n' && (flags & SMTP_GET_FLAG_SKIP)
+	&& vstream_feof(stream) == 0 && vstream_ferror(stream) == 0)
+	while ((next_char = VSTREAM_GETC(stream)) != VSTREAM_EOF
+	       && next_char != '\n')
+	     /* void */ ;
 
     /*
      * EOF is bad, whether or not it happens in the middle of a record. Don't
      * allow data that was truncated because of EOF.
      */
-    if (vstream_feof(stream) || vstream_ferror(stream)) {
-	if (msg_verbose)
-	    msg_info("smtp_get: EOF");
-	vstream_longjmp(stream, SMTP_ERR_EOF);
-    }
+    if (vstream_ftimeout(stream))
+	smtp_longjmp(stream, SMTP_ERR_TIME, "smtp_get");
+    if (vstream_feof(stream) || vstream_ferror(stream))
+	smtp_longjmp(stream, SMTP_ERR_EOF, "smtp_get");
     return (last_char);
 }
 
@@ -329,7 +382,7 @@ int     smtp_get(VSTRING *vp, VSTREAM *stream, ssize_t bound)
 
 void    smtp_fputs(const char *cp, ssize_t todo, VSTREAM *stream)
 {
-    ssize_t  err;
+    ssize_t err;
 
     if (todo < 0)
 	msg_panic("smtp_fputs: negative todo %ld", (long) todo);
@@ -340,23 +393,21 @@ void    smtp_fputs(const char *cp, ssize_t todo, VSTREAM *stream)
     smtp_timeout_reset(stream);
     err = (vstream_fwrite(stream, cp, todo) != todo
 	   || vstream_fputs("\r\n", stream) == VSTREAM_EOF);
-    smtp_timeout_detect(stream);
 
     /*
      * See if there was a problem.
      */
-    if (err != 0) {
-	if (msg_verbose)
-	    msg_info("smtp_fputs: EOF");
-	vstream_longjmp(stream, SMTP_ERR_EOF);
-    }
+    if (vstream_ftimeout(stream))
+	smtp_longjmp(stream, SMTP_ERR_TIME, "smtp_fputs");
+    if (err != 0)
+	smtp_longjmp(stream, SMTP_ERR_EOF, "smtp_fputs");
 }
 
 /* smtp_fwrite - write one string to SMTP peer */
 
 void    smtp_fwrite(const char *cp, ssize_t todo, VSTREAM *stream)
 {
-    ssize_t  err;
+    ssize_t err;
 
     if (todo < 0)
 	msg_panic("smtp_fwrite: negative todo %ld", (long) todo);
@@ -366,16 +417,14 @@ void    smtp_fwrite(const char *cp, ssize_t todo, VSTREAM *stream)
      */
     smtp_timeout_reset(stream);
     err = (vstream_fwrite(stream, cp, todo) != todo);
-    smtp_timeout_detect(stream);
 
     /*
      * See if there was a problem.
      */
-    if (err != 0) {
-	if (msg_verbose)
-	    msg_info("smtp_fwrite: EOF");
-	vstream_longjmp(stream, SMTP_ERR_EOF);
-    }
+    if (vstream_ftimeout(stream))
+	smtp_longjmp(stream, SMTP_ERR_TIME, "smtp_fwrite");
+    if (err != 0)
+	smtp_longjmp(stream, SMTP_ERR_EOF, "smtp_fwrite");
 }
 
 /* smtp_fputc - write to SMTP peer */
@@ -389,14 +438,12 @@ void    smtp_fputc(int ch, VSTREAM *stream)
      */
     smtp_timeout_reset(stream);
     stat = VSTREAM_PUTC(ch, stream);
-    smtp_timeout_detect(stream);
 
     /*
      * See if there was a problem.
      */
-    if (stat == VSTREAM_EOF) {
-	if (msg_verbose)
-	    msg_info("smtp_fputc: EOF");
-	vstream_longjmp(stream, SMTP_ERR_EOF);
-    }
+    if (vstream_ftimeout(stream))
+	smtp_longjmp(stream, SMTP_ERR_TIME, "smtp_fputc");
+    if (stat == VSTREAM_EOF)
+	smtp_longjmp(stream, SMTP_ERR_EOF, "smtp_fputc");
 }
