@@ -255,7 +255,8 @@ const char *psc_print_state_flags(int flags, const char *context)
 	"NEW", PSC_STATE_FLAG_NEW,
 	"BLIST_FAIL", PSC_STATE_FLAG_BLIST_FAIL,
 	"HANGUP", PSC_STATE_FLAG_HANGUP,
-	"CACHE_EXPIRED", PSC_STATE_FLAG_CACHE_EXPIRED,
+	/* unused */
+	"WLIST_FAIL", PSC_STATE_FLAG_WLIST_FAIL,
 
 	"PENAL_UPDATE", PSC_STATE_FLAG_PENAL_UPDATE,
 	"PENAL_FAIL", PSC_STATE_FLAG_PENAL_FAIL,
