@@ -87,6 +87,7 @@
 #include <sane_connect.h>
 #include <myaddrinfo.h>
 #include <sock_addr.h>
+#include <inet_proto.h>
 
 /* Global library. */
 
@@ -456,12 +457,12 @@ static void smtp_cache_policy(SMTP_STATE *state, const char *dest)
     state->misc_flags &= ~SMTP_MISC_FLAG_CONN_CACHE_MASK;
 
     /*
-     * XXX Disable connection caching when SASL authentication is
+     * XXX Disable connection caching when sender-dependent authentication is
      * enabled. We must not send someone elses mail over an authenticated
      * connection, and we must not send mail that requires authentication
      * over a connection that wasn't authenticated.
      */
-    if (var_smtp_sasl_passwd && *var_smtp_sasl_passwd)
+    if (var_smtp_sender_auth)
 	return;
 
     if (smtp_cache_dest && string_list_match(smtp_cache_dest, dest)) {
@@ -682,10 +683,10 @@ static int smtp_reuse_session(SMTP_STATE *state, int lookup_mx,
     return (session_count);
 }
 
-/* smtp_connect_remote - establish remote connection */
+/* smtp_connect_inet - establish network connection */
 
-static void smtp_connect_remote(SMTP_STATE *state, const char *nexthop,
-				        char *def_service)
+static void smtp_connect_inet(SMTP_STATE *state, const char *nexthop,
+			              char *def_service)
 {
     DELIVER_REQUEST *request = state->request;
     ARGV   *sites;
@@ -695,6 +696,16 @@ static void smtp_connect_remote(SMTP_STATE *state, const char *nexthop,
     int     retry_plain = 0;
     DSN_BUF *why = state->why;
 
+    /*
+     * For sanity, require that at least one of INET or INET6 is enabled.
+     * Otherwise, we can't look up interface information, and we can't
+     * convert names or addresses.
+     */
+    if (inet_proto_info()->ai_family_list[0] == 0) {
+	dsb_simple(why, "4.4.4", "all network protocols are disabled");
+	return;
+    }
+
     /*
      * First try to deliver to the indicated destination, then try to deliver
      * to the optional fall-back relays.
@@ -1036,7 +1047,7 @@ int     smtp_connect(SMTP_STATE *state)
 	} else {
 	    if (strncmp(destination, "inet:", 5) == 0)
 		destination += 5;
-	    smtp_connect_remote(state, destination, DEF_LMTP_SERVICE);
+	    smtp_connect_inet(state, destination, DEF_LMTP_SERVICE);
 	}
     }
 
@@ -1049,7 +1060,7 @@ int     smtp_connect(SMTP_STATE *state)
      * Postfix configurations that have a host with such a name.
      */
     else {
-	smtp_connect_remote(state, destination, DEF_SMTP_SERVICE);
+	smtp_connect_inet(state, destination, DEF_SMTP_SERVICE);
     }
 
     /*
