@@ -12,6 +12,11 @@
 /*	int	argc;
 /*	SMTPD_TOKEN *argv;
 /*
+/*	void	smtpd_sasl_auth_extern(state, username, method)
+/*	SMTPD_STATE *state;
+/*	const char *username;
+/*	const char *method;
+/*
 /*	void	smtpd_sasl_auth_reset(state)
 /*	SMTPD_STATE *state;
 /*
@@ -47,6 +52,13 @@
 /* .PP
 /*	smtpd_sasl_auth_reset() cleans up after the AUTH command.
 /*	This is required before smtpd_sasl_auth_cmd() can be used again.
+/*	This may be called even if SASL authentication is turned off
+/*	in main.cf.
+/*
+/*	smtpd_sasl_auth_extern() records authentication information
+/*	that is received from an external source.
+/*	This may be called even if SASL authentication is turned off
+/*	in main.cf.
 /*
 /*	smtpd_sasl_mail_opt() implements the SASL-specific AUTH=sender
 /*	option to the MAIL FROM command. The result is an error response
@@ -152,12 +164,6 @@ int     smtpd_sasl_auth_cmd(SMTPD_STATE *state, int argc, SMTPD_TOKEN *argv)
 	smtpd_chat_reply(state, "503 5.5.1 Error: authentication not enabled");
 	return (-1);
     }
-#define IN_MAIL_TRANSACTION(state) ((state)->sender != 0)
-    if (IN_MAIL_TRANSACTION(state)) {
-	state->error_mask |= MAIL_ERROR_PROTOCOL;
-	smtpd_chat_reply(state, "503 5.5.1 Error: MAIL transaction in progress");
-	return (-1);
-    }
     if (smtpd_milters != 0 && (err = milter_other_event(smtpd_milters)) != 0) {
 	if (err[0] == '5') {
 	    state->error_mask |= MAIL_ERROR_POLICY;
@@ -189,12 +195,8 @@ int     smtpd_sasl_auth_cmd(SMTPD_STATE *state, int argc, SMTPD_TOKEN *argv)
 	smtpd_chat_reply(state, "501 5.5.4 Syntax: AUTH mechanism");
 	return (-1);
     }
-
     /* Don't reuse the SASL handle after authentication failure. */
-#ifndef SMTPD_FLAG_AUTH_USED
-#define SMTPD_FLAG_AUTH_USED	(1<<15)
-#endif
-#ifndef XSASL_TYPE_CYRUS 
+#ifndef XSASL_TYPE_CYRUS
 #define XSASL_TYPE_CYRUS	"cyrus"
 #endif
     if (state->flags & SMTPD_FLAG_AUTH_USED) {
@@ -221,13 +223,6 @@ int     smtpd_sasl_auth_cmd(SMTPD_STATE *state, int argc, SMTPD_TOKEN *argv)
     return (smtpd_sasl_authenticate(state, auth_mechanism, initial_response));
 }
 
-/* smtpd_sasl_auth_reset - clean up after AUTH command */
-
-void    smtpd_sasl_auth_reset(SMTPD_STATE *state)
-{
-    smtpd_sasl_logout(state);
-}
-
 /* smtpd_sasl_mail_opt - SASL-specific MAIL FROM option */
 
 char   *smtpd_sasl_mail_opt(SMTPD_STATE *state, const char *addr)
@@ -236,10 +231,6 @@ char   *smtpd_sasl_mail_opt(SMTPD_STATE *state, const char *addr)
     /*
      * Do not store raw RFC2554 protocol data.
      */
-    if (!smtpd_sasl_is_active(state)) {
-	state->error_mask |= MAIL_ERROR_PROTOCOL;
-	return ("503 5.5.4 Error: authentication disabled");
-    }
 #if 0
     if (state->sasl_username == 0) {
 	state->error_mask |= MAIL_ERROR_PROTOCOL;
