@@ -65,16 +65,16 @@
 
 static const char *dict_default_lookup(DICT *dict, const char *unused_key)
 {
-    msg_fatal("%s table %s: lookup operation is not supported",
+    msg_fatal("table %s:%s: lookup operation is not supported",
 	      dict->type, dict->name);
 }
 
 /* dict_default_update - trap unimplemented operation */
 
-static void dict_default_update(DICT *dict, const char *unused_key,
-				        const char *unused_value)
+static int dict_default_update(DICT *dict, const char *unused_key,
+			               const char *unused_value)
 {
-    msg_fatal("%s table %s: update operation is not supported",
+    msg_fatal("table %s:%s: update operation is not supported",
 	      dict->type, dict->name);
 }
 
@@ -82,7 +82,7 @@ static void dict_default_update(DICT *dict, const char *unused_key,
 
 static int dict_default_delete(DICT *dict, const char *unused_key)
 {
-    msg_fatal("%s table %s: delete operation is not supported",
+    msg_fatal("table %s:%s: delete operation is not supported",
 	      dict->type, dict->name);
 }
 
@@ -91,7 +91,7 @@ static int dict_default_delete(DICT *dict, const char *unused_key)
 static int dict_default_sequence(DICT *dict, int unused_function,
 		         const char **unused_key, const char **unused_value)
 {
-    msg_fatal("%s table %s: sequence operation is not supported",
+    msg_fatal("table %s:%s: sequence operation is not supported",
 	      dict->type, dict->name);
 }
 
@@ -99,7 +99,7 @@ static int dict_default_sequence(DICT *dict, int unused_function,
 
 static void dict_default_close(DICT *dict)
 {
-    msg_fatal("%s table %s: close operation is not supported",
+    msg_fatal("table %s:%s: close operation is not supported",
 	      dict->type, dict->name);
 }
 
@@ -121,6 +121,9 @@ DICT   *dict_alloc(const char *dict_type, const char *dict_name, ssize_t size)
     dict->stat_fd = -1;
     dict->mtime = 0;
     dict->fold_buf = 0;
+    dict->owner.status = DICT_OWNER_UNKNOWN;
+    dict->owner.uid = ~0;
+    dict->error = DICT_ERR_NONE;
     return dict;
 }
 
