@@ -21,7 +21,8 @@
 
 extern int valid_hostname(const char *, int);
 extern int valid_hostaddr(const char *, int);
-extern int valid_hostliteral(const char *, int);
+extern int valid_ipv4_hostaddr(const char *, int);
+extern int valid_ipv6_hostaddr(const char *, int);
 
 /* LICENSE
 /* .ad
